SystemOrganization addCategory: #'SomethingIt-JavaScript-Socket'!

SIExternalInterpreter subclass: #SIJavaScript
	instanceVariableNames: 'socket inputSemaphore'
	classVariableNames: 'HttpServer Undefined'
	poolDictionaries: ''
	category: 'SomethingIt-JavaScript-Socket'!

!SIJavaScript class methodsFor: 'class initialization' stamp: 'tomooda 7/30/2013 15:17'!
defaultPage
	"The initial HTML5 page as a JavaScript engine"

	^'<html>
<head>
	<title>SIJavaScript</title>
	<script type="text/javascript" src="http://chirp.scratchr.org/dl/experimental/JsMorphic/morphic.js"></script>
	<script>
	window.onload = function () {
		var conn = {};
		var log = document.getElementById("log");
		var state = document.getElementById("status"); 
		SOMETHINGit = {};
		World = new WorldMorph(document.getElementById("world"));
		function loop() { World.doOneCycle();};
		setInterval(loop, 100);
 
		if (window.WebSocket === undefined) {
			state.innerHTML = "(WebSockets not supported)";
		} else {
			conn = new WebSocket("ws://localhost:8080/broadcast");
			conn.onopen = function () {
				state.innerHTML = "WebSocket: Connected";
			};
 
			conn.onmessage = function (event) {
				var input = event.data;
				var output;
				try {
					output = eval(input);
				} catch (e) {
					output = {"error": e};
				}
				try {
					output = JSON.stringify(output);
				} catch (e) {
					output = "Can''t JSON";
				}
				conn.send(output);
  		    		log.innerHTML = "<li>" + input + "<br>" + output + "</li>" + log.innerHTML;
   	 		};
    
    			conn.onclose = function (event) {
				state.innerHTML = "WebSocket: Closed";
			};
		};
	}
	</script> 
</head>
<body>
	<p id="status">WebSocket: Not connected</p> 
            <canvas id="world" width="800" height="600">
                <p>Your browser doesn''t support canvas.</p>
            </canvas>

	<ul id="log"></ul> 
</body>
</html>'! !

!SIJavaScript class methodsFor: 'examples' stamp: 'tomooda 7/30/2013 15:16'!
exampleGravity: aSIJavaScript 
	"SIJavaScript exampleGravity: self"
	| vdm points velocities add gravity |
	aSIJavaScript plainEvaluate: '
	point1 = new CircleBoxMorph();
	point2 = new CircleBoxMorph();
	point3 = new CircleBoxMorph();
	point1.setColor(new Color(255, 0, 0));
	point2.setColor(new Color(0, 255, 0));
	point3.setColor(new Color(0, 0, 255));
	point1.setExtent(new Point(10, 10));
	point2.setExtent(new Point(10, 10));
	point3.setExtent(new Point(10, 10));
	World.add(point1);
	World.add(point2);
	World.add(point3);
	World.changed();'.
	vdm := SIVDM specification: '
module Point
exports all
definitions
types
  Point :: x : real
           y : real;
functions
  plus : Point * Point -> Point
  plus(mk_Point(x1,y1), mk_Point(x2,y2)) == mk_Point(x1+x2, y1+y2);

  minus : Point * Point -> Point
  minus(mk_Point(x1,y1), mk_Point(x2,y2)) == mk_Point(x1-x2, y1-y2);

  neg : Point -> Point
  neg(mk_Point(x,y)) == mk_Point(-x, -y);

  scale : real * Point -> Point
  scale(a, mk_Point(x,y)) == mk_Point(a*x, a*y);

  dist2 : Point -> real
  dist2(mk_Point(x,y)) == x * x + y * y;

  dist : Point -> real
  dist(p) == dist2(p) ** 0.5;
end Point

module Gravity
exports all
imports from Point
    types
        Point renamed Point;
    functions
        plus renamed plus;
        minus renamed minus;
        neg renamed neg;
        scale renamed scale;
        dist2 renamed dist2;
        dist renamed dist
definitions
values
  gmn = 50.0;
functions
  total : seq of Point -> Point
  total(ps) == cases ps :
    [] -> mk_Point(0.0,0.0),
    others -> plus(hd ps, total(tl ps))
    end;

  gravity1 : Point * Point -> Point
  gravity1(p1, p2) ==
    let
      v = minus(p2, p1),
      d2 = dist2(v)
    in
      if d2 < 400.0
      then mk_Point(0.0, 0.0)
      else scale(gmn/(d2**1.5), v);

  gravity : seq of Point -> seq of Point
  gravity(ps) == [total([gravity1(ps(i1), ps(i2)) | i2 in set {1,...,len ps}]) | i1 in set {1,...,len ps}];
end Gravity

module ParticlesSimulation
exports all
imports
    from Point
        types
            Point renamed Point;
        functions
            plus renamed plus,
    from Gravity
        functions
            gravity renamed gravity

definitions
state Particles of
  positions : seq of Point
  velocities : seq of Point
init
  particles == particles = mk_Particles([], [])
end
operations
  Step : () ==> ()
  Step() == (
    velocities := (let forces = gravity(positions) in [plus(velocities(i), forces(i)) | i in set {1,...,len forces}]);
    positions := [plus(positions(i), velocities(i)) | i in set {1,...,len velocities}]);
  GetPositions : () ==> seq of Point
  GetPositions() == return positions;
  Init : seq of Point * seq of Point ==> ()
  Init(ps, vs) == (positions := ps; velocities := vs);
end ParticlesSimulation
'.
	vdm printer
		addClassMap: #Point
		to: [:point :vdmStream | 
			vdmStream nextPutAll: 'mk_Point`Point('.
			point x printOn: vdmStream.
			vdmStream nextPut: $,.
			point y printOn: vdmStream.
			vdmStream nextPut: $)].
	vdm parser
		addAlgebraicDataMap: 'mk_Point'
		to: [:constructor :args | args first @ args second].
	gravity := 'Gravity`gravity' asVDMFunctionIn: vdm.
	points := {100.0 @ 250.0. 250.0 @ 250.0. 250.0 @ 350.0} asOrderedCollection.
	velocities := {0.0 @ 0.0. 0.0 @ 0.0. 0.0 @ 0.0} asOrderedCollection.
	add := [:array1 :array2 | 1
				to: array1 size
				do: [:index | array1 at: index put: (array1 at: index)
							+ (array2 at: index)]].
	1000
		timesRepeat: [| forces code |
			forces := gravity value: points.
			add value: velocities value: forces.
			add value: points value: velocities.
			code := String new writeStream.
			code nextPutAll: 'point1.setPosition(new Point('.
			points first x asInteger printOn: code.
			code nextPutAll: ', '.
			points first y asInteger printOn: code.
			code nextPutAll: ')); point1.changed();'.
			code cr.
			code nextPutAll: 'point2.setPosition(new Point('.
			points second x asInteger printOn: code.
			code nextPutAll: ', '.
			points second y asInteger printOn: code.
			code nextPutAll: ')); point2.changed();'.
			code cr.
			code nextPutAll: 'point3.setPosition(new Point('.
			points third x asInteger printOn: code.
			code nextPutAll: ', '.
			points third y asInteger printOn: code.
			code nextPutAll: ')); point3.changed();'.
			code cr.
			aSIJavaScript plainEvaluate: code contents]! !

!SIJavaScript class methodsFor: 'class initialization' stamp: 'tomooda 6/18/2013 13:09'!
initialize
	super initialize.
	Undefined ifNil: [ Undefined := Object new]! !

!SIJavaScript class methodsFor: 'class initialization' stamp: 'tomooda 8/5/2013 14:34'!
initializeHttpServer
	"SIJavaScript initializeHttpServer"
	
	HttpServer ifNotNil: [HttpServer destroy].
	HttpServer := WebServer new.
	HttpServer listenOn: 8080.
	HttpServer addService: '/' action:[:req|
		"Serves the main html as a JavaScript engine"
		req send200Response: self defaultPage contentType: 'text/html'.
	].
	HttpServer addService: '/broadcast' action:[:req|
		| socket interpreter |
		"Implements the WebSocket broadcaster"
		socket := req asWebSocket.
		socket timeout: 0.
		interpreter := self on: socket.
		interpreter inspect.
		socket run].! !

!SIJavaScript class methodsFor: 'instance creation' stamp: 'tomooda 6/18/2013 09:24'!
on: aWebSocket
	^ self new setWebSocket: aWebSocket; yourself! !

!SIJavaScript class methodsFor: 'constants' stamp: 'tomooda 6/18/2013 13:09'!
undefined
	^ Undefined! !

!SIJavaScript methodsFor: 'initialize-release' stamp: 'tomooda 6/18/2013 09:36'!
basicClose
	super basicClose.
	socket
		ifNotNil: [:s | 
			socket := nil.
			s close]! !

!SIJavaScript methodsFor: 'communicating' stamp: 'tomooda 6/18/2013 10:22'!
expression: aString ifError: errorReceivingBlock 
	^ aString! !

!SIJavaScript methodsFor: 'initialize-release' stamp: 'tomooda 6/18/2013 10:04'!
initialize
	super initialize.
	inputSemaphore := Semaphore new! !

!SIJavaScript methodsFor: 'communicating' stamp: 'tomooda 6/18/2013 10:04'!
readUpToPrompt
	inputSemaphore wait.
	^ reader! !

!SIJavaScript methodsFor: 'communicating' stamp: 'tomooda 6/18/2013 10:06'!
readUpToPrompt: anIntegerOrNil
	^ self readUpToPrompt! !

!SIJavaScript methodsFor: 'communicating' stamp: 'tomooda 6/18/2013 09:41'!
send: aString
	socket send: aString! !

!SIJavaScript methodsFor: 'private' stamp: 'tomooda 6/18/2013 10:05'!
setWebSocket: aWebSocket
	socket := aWebSocket.
	socket onClose: [self close].
	socket onMessage: [:data | reader := data. inputSemaphore signal ]! !

!SIJavaScript methodsFor: 'private' stamp: 'tomooda 6/18/2013 10:07'!
startupProcess
	"do nothing"! !

!SIJavaScript methodsFor: 'constant access' stamp: 'tomooda 6/18/2013 13:09'!
undefined
	^ self class undefined! !

Object subclass: #SIJavaScriptValueParser
	instanceVariableNames: 'parser'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SomethingIt-JavaScript-Socket'!

!SIJavaScriptValueParser methodsFor: 'initialize-release' stamp: 'tomooda 7/30/2013 13:13'!
initialize
	| digit odigit int octal hex float escChar string bool null undefined array object |
	super initialize.
	digit := #digit asParser.
	odigit := PPPredicateObjectParser chars: '01234567' message: 'octal expected'.
	int := ($+ asParser / $- asParser) optional, digit plus ==> [:pair |
		| abs |
		abs := pair second inject: 0 into: [ :i :c | i * 10 + c digitValue].		
		pair first = $- ifTrue: [abs negated] ifFalse: [abs]].
	octal := ($+ asParser / $- asParser) optional, $0 asParser, odigit plus ==> [:triple |
		| abs |
		abs := triple third inject: 0 into: [ :i :c | i * 8 + c digitValue].		
		triple first = $- ifTrue: [abs negated] ifFalse: [abs]].
	hex := ($+ asParser / $- asParser) optional, '0x' asParser, #hex asParser plus ==> [:triple |
		| abs |
		abs := triple second inject: 0 into: [ :i :c | i * 16 + c digitValue].		
		triple first = $- ifTrue: [abs negated] ifFalse: [abs]].
	float := int, $. asParser, digit plus, (($e asParser / $E asParser), int) optional ==> [:quadraple |
		| order num |
		order := 1.0.
		num := quadraple third inject: quadraple first asFloat into: [ :n :c | c digitValue * (order := order * 0.1) + n].
		quadraple fourth ifNotNilDo: [ :pair | num := num * (10.0 raisedTo: pair second)].
		num].
	escChar := $b asParser ==> [:s | Character backspace ] / ($f asParser ==> [:s | Character newPage]) / ($n asParser ==> [ :s | Character cr] ) / ($r asParser ==> [:s | Character lf]) / ($t asParser ==> [:s | Character tab]) / ($v asParser ==> [:s | Character value: 11]) / (odigit, odigit, odigit ==> [:triple | Character value: (triple first digitValue * 64) + (triple second digitValue * 8) + triple third digitValue]) / ($x asParser, #hex asParser, #hex asParser ==> [:triple | Character value: triple second digitValue * 16 + triple third digitValue]) / ($u asParser, #hex asParser, #hex asParser, #hex asParser, #hex asParser ==> [ :pentaple | Character value: (pentaple second digitValue * 4096) + (pentaple third digitValue * 256) + (pentaple fourth digitValue * 16) + (pentaple fifth digitValue)]) / (#any asParser ==> [:c | c]).
	bool := 'true' asParser ==> [ :s | true] / ('false' asParser ==> [:s | false]).
	null := 'null' asParser ==> [ :s | nil].
	undefined := 'undefined' asParser ==> [:s | SIJavaScript undefined ].
	string := ($" asParser, (($" asParser not, (($\ asParser, escChar ==> [ :pair | pair second]) / #any asParser)) star ==> [ :chars | String withAll: (chars collect: [ :pair | pair last])]), $" asParser) ==> [ :triple | triple second] / (($' asParser, (($' asParser not, (($\ asParser, escChar ==> [ :pair | pair second]) / #any asParser)) star ==> [ :chars | String withAll: (chars collect: [ :pair | pair last])]), $' asParser) ==> [ :triple | triple second]).
	array := PPUnresolvedParser new.
	object := PPUnresolvedParser new.
	parser := PPUnresolvedParser new.
	array def: $[ asParser, parser, ($, asParser, parser ==> [ :pair | pair second]) star, $] asParser ==> [:quad | | tail a | tail := quad third. a := Array new: tail size + 1. a at: 1 put: quad second. 1 to: tail size do: [ :i | a at: i + 1 put: (tail at: i)]. a].
	object def: ${ asParser, (string trim, $: asParser, parser, ($, asParser, string trim, $: asParser, parser) star) optional, $} asParser ==> [:triple | triple second ifNotNil: [:quad | quad fourth inject: (Dictionary new at: quad first put: quad third; yourself) into: [ :d :q | d at: q second put: q fourth; yourself] ] ifNil: [Dictionary new]].
	parser def: (string / bool / null / undefined / array / object / hex / octal / float / int) trim.! !

!SIJavaScriptValueParser methodsFor: 'parsing' stamp: 'tomooda 6/18/2013 11:48'!
parseStream: aStream ifError: errorBlock
	^ parser parse: aStream onError: [errorBlock value: 'Invalid value: ', aStream contents]! !

Object subclass: #SIWebSocketInputStream
	instanceVariableNames: 'buffer semaphore lock empty'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SomethingIt-JavaScript-Socket'!

!SIWebSocketInputStream methodsFor: 'accessing' stamp: 'tomooda 6/18/2013 10:00'!
basicNext
	^ lock
		critical: [buffer isEmpty
				ifTrue: [empty]
				ifFalse: [| first |
					first := buffer first.
					buffer copyFrom: 2 to: buffer size.
					first]]! !

!SIWebSocketInputStream methodsFor: 'initialize-release' stamp: 'tomooda 6/18/2013 09:59'!
initialize
	super initialize.
	semaphore := Semaphore new.
	lock := Semaphore forMutualExclusion.
	buffer := Array new.
	empty := Object new! !

!SIWebSocketInputStream methodsFor: 'accessing' stamp: 'tomooda 6/18/2013 10:02'!
next
	| next |
	[next := self basicNext.
	next = empty]
		whileTrue: [semaphore wait].
	^ next! !

!SIWebSocketInputStream methodsFor: 'accessing' stamp: 'tomooda 6/18/2013 09:53'!
nextPut: aString 
	lock
		critical: [buffer := buffer copyWith: aString.
			buffer size = 1
				ifTrue: [semaphore signal]]! !
SIJavaScript initialize!
